const express = require('express');
require('dotenv').config();
const app = express();
const mongoose = require('mongoose');
const Consommation = require('../models/consommation');
const mongoURI = process.env.MONGO_URI || 'mongodb://localhost/abreuvoir';

const port = 3000;


app.use(express.json());

const path = require('path');
const viewsPath = path.join(__dirname, '../views');
app.use(express.static(viewsPath));

// Connexion à MongoDB
mongoose.connect(mongoURI, { useNewUrlParser: true, useUnifiedTopology: true })
    .then(() => console.log('MongoDB Connected'))
    .catch(err => {
        console.error('MongoDB connection error:', err.message);
    });


// Constantes
const rayonSeau = 15; // Le rayon du seau en cm
const hauteurInitialeEau = 50; // la hauteur initiale de l'eau en cm
const distanceSupplementaire = 5; // la distance supplémentaire en cm entre le capteur et le bord supérieur du seau
let volumeEauInitial = Math.PI * rayonSeau**2 * hauteurInitialeEau; // Calcul du volume d'eau initial en cm^3 (mL)


app.get('/', (req, res) => {
    res.sendFile(path.join(viewsPath, 'index.html'));
});

app.get('/information-generale', (req, res) => {
    res.sendFile(path.join(viewsPath, 'dashboard.html'));
});

app.get('/graphique-consommations', (req, res) => {
    res.sendFile(path.join(viewsPath, 'chart.html'));
});

app.post('/distance', (req, res) => {
    const { distance } = req.body;
    console.log(`Distance reçue: ${distance} cm`);
    res.status(200).json({ message: 'Distance reçue avec succès' });
});

// Route pour définir/mettre à jour le volume d'eau initial
app.post('/definir-volume-initial', (req, res) => {
    const { hauteurEau } = req.body; // Hauteur d'eau fournie par l'utilisateur
    volumeEauInitial = Math.PI * rayonSeau**2 * hauteurEau; // Mise à jour du volume d'eau initial
    res.json({ message: 'Volume d\'eau initial mis à jour', volumeEauInitial });
});

// Route pour calculer le volume d'eau et la consommation
app.post('/calculer-volume', async (req, res) => {
    let { distanceCapteur } = req.body;

    distanceCapteur = parseInt(distanceCapteur);

    if (typeof distanceCapteur !== 'number' || distanceCapteur < 0) {
        return res.status(400).json({ message: 'Distance invalide' });
    }

    const distanceReelle = distanceCapteur - distanceSupplementaire;
    if (distanceReelle < 0) {
        return res.status(400).json({ message: 'Distance réelle invalide' });
    }

    const hauteurEau = hauteurInitialeEau - distanceReelle;
    const volumeEau = Math.PI * rayonSeau ** 2 * hauteurEau;
    const consommationEau = volumeEauInitial - volumeEau;

    try {
        const consom = new Consommation({
            distance: distanceReelle,
            volume: volumeEau,
            consommation: consommationEau
        });

        await consom.save();

        res.status(201).json({
            consom,
            calculs: {
                hauteurEau,
                volumeEau,
                consommationEau
            }
        });
    } catch (error) {
        res.status(500).json({ message: error.message });
    }
});

app.get('/consommations', async (req, res) => {
    try {
        const consommations = await Consommation.find();
        res.json(consommations);
    } catch (error) {
        res.status(500).json({ message: error.message });
    }
});

app.listen(port, () => {
    console.log(`Server listening at http://localhost:${port}`);
});
